<cfsetting enablecfoutputonly="yes">

<!--- --------------------------------------------------------------- 	
	@author:		Kevin Miller
	@date:		5/9/2005
	@version:	1.0 
	
	@description: 	
   		GLOBAL cfc argument template
		
	@changelog: 
		[version - date - author - comment]
		1.0.0 - 5/9/2005 - Kevin Miller - creation 
------------------------------------------------------------------ --->

<!--- 
	NOTES:
	This template is included in the detail page of DBX.  As such it has a query 
	named "getobject" exposed to the page scope, and it is this query that is 
	used in "generating" the cfc arguments. 
	
	The argument generator template simply builds up a big string which is then 
	written into a hidden input on the detail page in DBX; when the Generate CFC Args 
	button is clicked, its a simple form post to a popup window that displays the 
	form input in question.  Therefore, the code is "generated" on the detail page, 
	not when the popup window is opened, for whatever thats worth.
	
	The argument generator templates do only that: generate the cfargument code for the object 
	in question.  That is its only purpose, and is not intended to be a 'cfc' generator, since to do 
	so would be to impose my own styles on others, which I don't think is prudent.  Regardless,
	these templates will reduce some drudge and automate the cfargument code generation.
	
	getobject query columns: (listing the most relevant columns, others exist but aren't used)
	-----------------------------------
		TABLE_OWNER : table owner name
		TABLE_NAME : table name
		COLUMN_NAME : name of each column
		DATA_TYPE : integer code for odbc datatype (recommended to use TYPE_NAME below)
		TYPE_NAME : string representing the datatype; convert to cf_sql_type with function getCFSQLType()
		PRECISION : number of significant digits, base 10
		LENGTH : transfer size of the data
		SCALE : number of digits to the right of the decimal point
		RADIX : base for numeric datatypes 
		NULLABLE : whether the column accepts null values
		COLUMN_DEF : default value of the column
	
 --->
 
<cfscript>
	arg = structnew(); // will hold all the template values 
		arg.header = ""; // header block for comments 
		arg.body = ""; // the body of the code block 
		
		arg.content = ""; // this is the final built up content comprised of the above struct keys concatenated
</cfscript>

<cfscript>
	// HEADER BLOCK
	arg.header = '
&lt;!--- 
NOTE: 
these arguments are generated without regard to query type, and are generated 
based upon detected columns/variables for the object in question. 

this cfargument generator is customizable, refer to the documentation for more details
---&gt;';
</cfscript>

<!--- VARS AND BODY BLOCKS --->
<cfoutput query="getobject">

	<cfscript>
		theVar = "#replace(replace(column_name,"_","","all"),"@","","all")#"; // remove var char; the name of the input var (i like to use column_name with underscores removed)
		theArgType = ""; // the argument type declaration
		theArgTypeDefault = ""; // default value, read from table columns, sprocs do not return this value
		theArgTypeRequired = ""; // used for required= attribute
		
		forceUnrequired = false;
		// when dealing with procs, all 'columns' return nullable
		if(isdefined("url.sp")) {
			forceUnrequired = true;
		}

		if(listfindnocase("char,nchar,varchar,nvarchar,text,ntext",type_name)) { // string type vars
			theArgType = "string"; 
			theArgTypeRequired = "yes";
			if(forceUnrequired or nullable) {
				// theArgTypeDefault = column_def;
				theArgTypeDefault = rereplace(column_def,'([(''|'')])','','all');
				theArgTypeRequired = "no";
			} 
		}
		else if (listfindnocase("int,int identity,smallint,tinyint,bigint,bit,money,decimal,numeric",type_name)) { // numeric types
			theArgType = "numeric"; 
			theArgTypeRequired = "yes";
			if(forceUnrequired or nullable) {
				theArgTypeDefault = rereplace(column_def,'([(|)])','','all');
				theArgTypeRequired = "no";
			} 
		}
		else if (listfindnocase("datetime,smalldatetime",type_name)) { // date types
			theArgType = "date"; 
			theArgTypeRequired = "yes";
			if(forceUnrequired or nullable) {
				theArgTypeDefault = rereplace(column_def,'([(''|'')])','','all');
				theArgTypeDefault = rereplace(theArgTypeDefault,'([(|)])','','all');
					if(theArgTypeDefault eq "getdate") {
						theArgTypeDefault = "##now()##";
					}
				theArgTypeRequired = "no";
			} 
		}
		else if (type_name eq "uniqueidentifier") {
			theArgType = "guid"; 
			theArgTypeRequired = "yes";
			if(forceUnrequired or nullable) {
				theArgTypeDefault = rereplace(column_def,'([(|)])','','all');
					if(theArgTypeDefault eq "newid") {
						theArgTypeDefault = "##createuuid()##";
					}
				theArgTypeRequired = "no";
			} 
		}
		else {
			theArgType = "any"; 
			theArgTypeRequired = "yes";
			if(forceUnrequired or (len(column_def) and nullable)) {
				theArgTypeDefault = column_def;
				theArgTypeRequired = "no";
			} 
		}
		
		if(theVar neq "returnvalue" and not listfindnocase("timestamp",type_name)) { // we dont' want either of these in the arguments
			arg.body = arg.body & '&lt;c' & 'fargument name="#theVar#" type="#theArgType#" required="#theArgTypeRequired#"';
			if(theArgTypeRequired eq "no") {
				arg.body = arg.body & ' default="#theArgTypeDefault#"';
			}
			arg.body = arg.body & '&gt;' & chr(10);
		}

	</cfscript>
	
</cfoutput>

<cfscript>
	// the final concatenated argument block 
	arg.content = htmleditformat(arg.header & chr(10) & chr(10) & arg.body);
</cfscript>

<cfsetting enablecfoutputonly="no">