<cfsetting enablecfoutputonly="yes">


<!--- --------------------------------------------------------------- 	
	@author:		Kevin Miller
	@date:		5/9/2005
	@version:	1.0 
	
	@description: 	
   		UPDATE stored proc template
		assumes int identity primary key
		
	@changelog: 
		[version - date - author - comment]
		1.0.0 - 5/9/2005 - Kevin Miller - creation 
------------------------------------------------------------------ --->

<!--- 
	NOTES:
	This template is included in the detail page of DBX.  As such it has a query 
	named "getobject" exposed to the page scope, and it is this query that is 
	used in "generating" the stored proc code. 
	
	The stored proc generator template simply builds up a big string which is then 
	written into a hidden input on the detail page in DBX; when the Generate Procs 
	button is clicked, its a simple form post to a popup window that displays the 
	form input in question.  Therefore, the code is "generated" on the detail page, 
	not when the popup window is opened, for whatever thats worth.
	
	getobject query columns: (listing the most relevant columns, others exist but aren't used)
	-----------------------------------
		TABLE_OWNER : table owner name
		TABLE_NAME : table name
		COLUMN_NAME : name of each column
		DATA_TYPE : integer code for odbc datatype (recommended to use TYPE_NAME below)
		TYPE_NAME : string representing the datatype; convert to cf_sql_type with function getCFSQLType()
		PRECISION : number of significant digits, base 10
		LENGTH : transfer size of the data
		SCALE : number of digits to the right of the decimal point
		RADIX : base for numeric datatypes 
		NULLABLE : whether the column accepts null values
		COLUMN_DEF : default value of the column
	
 --->
 
<cfscript>
	sp = structnew(); // will hold all the template values 
		sp.procname = ""; // name of the procedure
		sp.header = ""; // the opening code
		sp.vars = ""; // the input vars
		sp.comments = ""; // comment block for proc
		sp.validation = ""; // code for validating input values
		sp.body = ""; // the body of the proc code
		sp.body2 = ""; // the body of the proc code, part 2 (used when code must be generated 'split', as in insert statements)
		sp.footer = ""; // the closing code
		
		sp.content = ""; // this is the final built up content comprised of the above struct keys concatenated
</cfscript>

<cfscript>
	/* 
	PROC NAME PREPARATION CODE
	-------------------------------------------
	the following  is just a block i use to 'clean up' my table names for use in the naming the procedure 
	if they have underscores in them and to provide lowercase verb / uppercase noun.  for example: 
	
	if the table name was 'user_accounts', then the block of code below would feed the value 
	UserAccounts to the procname key in the struct, yielding something along the lines of: up_addUserAccounts 
	
	if you don't care and want to just use your tablename as is, uncomment the following line and remove the 
	'cleanup' block below:
	*/ 
	
	// ------- CODE FOR UNMODIFIED TABLENAME USE ------
	// sp.procname = "up_update" & getobject.table_name;

	// ------- MY TABLENAME CLEANUP CODE -----------
		procNameList = lcase(replace(getobject.table_name,"_"," ","all"));
		for(i=1; i lte listlen(procNameList," "); i = i +1) {
			if(len(listgetat(procNameList,i," ")) eq 1) { sp.procname = sp.procname & ucase(mid(listgetat(procNameList,i," "),1,1)); }
			else { sp.procname = sp.procname & ucase(mid(listgetat(procNameList,i," "),1,1)) & right(listgetat(procNameList,i," "),(len(listgetat(procNameList,i," "))-1)); }
		}
		sp.procname = "up_update" & sp.procname; 
	// --------------------------------------------------------------
		
		
	// HEADER BLOCK 
	sp.header = "CREATE PROCEDURE #sp.procname# "; 
	
	// FOOTER BLOCK 
	sp.footer = "RETURN 0" & chr(10) & chr(10) & "GO";
		
	// COMMENT BLOCK 
	sp.comment = "
  /*
  Object: #sp.procname#
  Description: Updates an entry in the #getobject.table_name# table 
  Returns: 
    resultset with number of rows affected
  
  Created by: DBX #dbx.version#
  Version: 1.0
  Created: #dateformat(now(),'mm/dd/yyyy')#
  Last changed: #dateformat(now(),'mm/dd/yyyy')#
  */
  ";

</cfscript>

<!--- VARS AND BODY BLOCKS --->
<cfoutput query="getobject">

	<cfscript>
		theVar = "@#replace(column_name,"_","","all")#"; // the name of the input var (i like to use column_name with underscores removed)
		theType = ""; // the variable type declaration
		theTypeDefault = ""; // default value
		theTypeValidation = ""; // used for error checking

		if(listfindnocase("char,nchar,varchar,nvarchar",type_name)) { // string type vars
			theType = "#type_name#(#precision#)"; 
			theTypeDefault = "NULL"; 
			if(not nullable) {
				theTypeValidation = "IF #theVar# IS NULL OR LEN(#theVar#) = 0 RETURN -1";
			}
		}
		else if (listfindnocase("int,int identity,smallint,tinyint,bigint,bit",type_name)) { // numeric types
			theType = "#type_name#"; 
			if(findnocase(" identity",theType)) {
				theType = replacenocase(type_name," identity","");
			}
			theTypeDefault = "NULL"; 
			if(not nullable) {
				theTypeValidation = "IF #theVar# IS NULL RETURN -1";
			}
		}
		else if (listfindnocase("datetime,smalldatetime",type_name)) { // date types
			theType = "#type_name#"; 
			theTypeDefault = "NULL"; 
			theTypeValidation = "IF #theVar# IS NOT NULL AND ISDATE(#theVar#) = 0 RETURN -1";
			if(not nullable) {
				theTypeValidation = theTypeValidation & chr(10) & "IF #theVar# IS NULL RETURN -1";
			}
		}
		else {
			theType = "#type_name#";
			theTypeDefault = "NULL"; 
		}
		
		if(not listfindnocase("timestamp",type_name)) { // we dont' want this in an update proc 
			sp.vars = sp.vars & "  " & theVar & " " & theType & " = " & theTypeDefault & "," & chr(10);
			if(type_name neq "int identity") {
				sp.body = sp.body & "  " & "#column_name# = #theVar#" & "," & chr(10); 
			}
			if(len(theTypeValidation)) {
				sp.validation = sp.validation & theTypeValidation & chr(10);
			}
		}
		
		if(listfindnocase(valuelist(getobject.type_name),"int identity")) { // its a table with an int identity 
			if(type_name eq "int identity") {
				sp.body2 = "WHERE #column_name# = #theVar#";
			}
		}
		else { // no int identity, use other columns 
			if(type_name neq "timestamp") {
				sp.body2 = sp.body2 & " AND #column_name# = #theVar#" & chr(10);
			}
		}
	</cfscript>
	
</cfoutput>

<cfscript>
	// remove trailing "," from each block
	sp.vars = removechars(sp.vars,(len(sp.vars)-1),1);
	sp.body = "UPDATE #getobject.table_name# " & chr(10) & "SET " & chr(10) & removechars(sp.body,(len(sp.body)-1),1);

	// code to return the rows affected  
	if(listfindnocase(valuelist(getobject.type_name),"int identity")) {
		sp.body2 = sp.body2 & chr(10) & chr(10) & "SELECT rowsAffected = @@rowcount -- return the number of affected rows"; 
	}
	else {
		sp.body2 = "WHERE 1=1" & chr(10) & sp.body2; 
	}
	
	// the final concatenated proc code 
	sp.content = sp.header & chr(10) & sp.comment & chr(10) &sp.vars & chr(10) & "AS" & chr(10) & chr(10) & sp.validation & chr(10) & sp.body & chr(10) & sp.body2 & chr(10) & chr(10) & sp.footer;
</cfscript>

<cfsetting enablecfoutputonly="no">