<cfsetting enablecfoutputonly="yes">

<!--- --------------------------------------------------------------- 	
	@author:		Kevin Miller
	@date:		5/8/2006
	@version:	1.0 
	
	@description: 	
   		basic xml argument template
		
	@changelog: 
		[version - date - author - comment]
		1.0.0 - 5/8/2006 - Kevin Miller - creation 
------------------------------------------------------------------ --->

<!--- 
	NOTES:
	This template is included in the detail page of DBX.  As such it has a query 
	named "getobject" exposed to the page scope, and it is this query that is 
	used in "generating" the cfc arguments. 
	
	The xml generator template simply builds up a big string which is then 
	written into a hidden input on the detail page in DBX; when the Generate XML 
	button is clicked, its a simple form post to a popup window that displays the 
	form input in question.  Therefore, the code is "generated" on the detail page, 
	not when the popup window is opened, for whatever thats worth.
	
	The xml generator templates do only that: generates xml code for the object 
	in question.  That is its only purpose, and based upon individual needs, it can/should
	be customized accordingly.   
	
	getobject query columns: (listing the most relevant columns, others exist but aren't used)
	-----------------------------------
		TABLE_OWNER : table owner name
		TABLE_NAME : table name
		COLUMN_NAME : name of each column
		DATA_TYPE : integer code for odbc datatype (recommended to use TYPE_NAME below)
		TYPE_NAME : string representing the datatype; convert to cf_sql_type with function getCFSQLType()
		PRECISION : number of significant digits, base 10
		LENGTH : transfer size of the data
		SCALE : number of digits to the right of the decimal point
		RADIX : base for numeric datatypes 
		NULLABLE : whether the column accepts null values
		COLUMN_DEF : default value of the column
	
 --->
 
<cfscript>
	xm = structnew(); // will hold all the template values 
		xm.header = ""; // header block for comments 
		xm.body = ""; // the body of the code block 
		xm.footer = ""; // the footer block of code
		
		xm.content = ""; // this is the final built up content comprised of the above struct keys concatenated
</cfscript>

<!--- header and footer for the component --->
<cfset xm.header = '&lt;?xml version="1.0" encoding="UTF-8"?&gt;'>
<cfset xm.header = listappend(xm.header,'#chr(9)#&lt;#getobject.table_name#&gt;',chr(10))>
<cfset xm.footer = '#chr(9)#&lt;/#getobject.table_name#&gt;'>

<!--- OUTPUT SAMPLE: 
		<companyid>
			<label>companyid</label>
			<key>pk</key>
			<maxlength>0</maxlength>
			<sortOrder/>
			<requiredInd>true</requiredInd>
			<notProvided>The companyid field is required but was not provided.</notProvided>
			<sqlDataType>CF_SQL_INTEGER</sqlDataType>
			<invalidType>The companyid field does not contain valid data.  This field must be a numeric value.</invalidType>
			<defaultErrorMessage>You must provide a valid company id in order to edit the company.</defaultErrorMessage>
			<toolTip></toolTip>
		</companyid>
 --->

<!--- VARS AND BODY BLOCKS --->
<cfset xmsortcounter = 0><!--- we don't want pk's in the sort --->

<cfoutput query="getobject">

	<!--- pick up key info by querying getobject_keys query existing in parent page --->
	<cfset nodeKeyType = "">
	<cfif getobject_keys.recordcount>
		<cfset goColname = column_name>
		<cfloop query="getobject_keys">
			<cfscript>
				for ( i=1; i lte 16; i = i+1 ) {
					if( trim(getobject_keys["cKeyCol#i#"][currentrow]) eq goColname ) {
						nodeKeyType = "pk"; 
						for ( y=1; y lte 16; y = y+1 ) {
							if( len(trim(getobject_keys["cRefCol#y#"][currentrow])) ) { 
								nodeKeyType = "fk";
								break;
							}
						}
						break;
					}
				}
			</cfscript>
		</cfloop>
		
	</cfif>	
	
	<cfscript>
	
		// this is just a list so we can output the struct contents 'in order'
		nodelist = "label,key,maxlength,sortorder,requiredind,notprovided,sqldatatype,invalidtype,invalidlength,defaulterrormessage,tooltip,formfieldtype,formdatatype,defaultvalue"; 
		
		node = structnew(); 
		
			node.label.name = "label"; 
			node.label.value = "#column_name#";
			node.label.forcetagpair = false; // whether to force open/closing tags when empty
			
			node.key.name = "key"; 
			node.key.value = nodeKeyType; // pk/fk/blank 
			node.key.forcetagpair = false; 
			
			node.maxlength.name = "maxlength"; 
			node.maxlength.value = ""; // overwritten below based upon type 
			node.maxlength.forcetagpair = false; 
			
			node.sortorder.name = "sortOrder"; 
			node.sortorder.value = ""; // overwritten below based upon type 
			node.sortorder.forcetagpair = false; 
			
			node.requiredind.name = "requiredInd"; 
			node.requiredind.value = iif(nullable,de("false"),de("true")); 
			node.requiredind.forcetagpair = false; 
			
			node.notprovided.name = "notProvided"; 
			node.notprovided.value = "The #column_name# field is required but was not provided."; 
			node.notprovided.forcetagpair = false; 
			
			node.sqldatatype.name = "sqlDataType"; 
			node.sqldatatype.value = getCFSQLType(type_name); 
			node.sqldatatype.forcetagpair = false; 
			
			node.invalidtype.name = "invalidType";
			node.invalidtype.value = ""; // overwritten below based upon type 
			node.invalidtype.forcetagpair = false; 
			
			node.invalidlength.name = "invalidLength"; 
			node.invalidlength.value = ""; // overwritten below based upon type 
			node.invalidlength.forcetagpair = false; 
			 
			node.defaulterrormessage.name = "defaultErrorMessage";
			node.defaulterrormessage.value = "You must provide a valid #column_name# in order to edit the #table_name#."; 
			node.defaulterrormessage.forcetagpair = false; 
			
			node.tooltip.name = "toolTip";
			node.tooltip.value = "";
			node.tooltip.forcetagpair = true; 
			 
			node.formfieldtype.name = "formFieldType";
			node.formfieldtype.value = "";
			node.formfieldtype.forcetagpair = true; 
			 
			node.formdatatype.name = "formDataType"; 
			node.formdatatype.value = "";
			node.formdatatype.forcetagpair = true; 
			 
			node.defaultvalue.name = "defaultValue";
			node.defaultvalue.value = ""; 
			node.defaultvalue.forcetagpair = true; 
			
		// string type vars			
		if(listfindnocase("char,nchar,varchar,nvarchar,text,ntext",type_name)) { 		
			xmsortcounter = xmsortcounter + 1; // increment the sort
			node.sortorder.value = xmsortcounter; 
			node.invalidtype.value = "The #column_name# is not valid.";
			node.invalidlength.value = "The #column_name# field is too long.  This field must be no more than #precision# character(s)."; 		
			node.maxlength.value = precision; 	
		} 
		
		// numeric types	
		else if (listfindnocase("int,int identity,smallint,smallint identity,tinyint,bigint,bigint identity,bit,money,decimal,numeric",type_name)) { 
			node.invalidtype.value = "The #column_name# does not contain valid data.  This field must be a numeric value.";
			node.maxlength.value = 0; 
			if(node.key.value neq "pk") { 
			xmsortcounter = xmsortcounter + 1; // increment the sort
			node.sortorder.value = xmsortcounter; 			
			}
		}
		
		// date types
		else if (listfindnocase("datetime,smalldatetime",type_name)) { 
			xmsortcounter = xmsortcounter + 1; // increment the sort
			node.sortorder.value = xmsortcounter; 
			node.invalidtype.value = "The #column_name# does not contain valid data.  This field must be a valid date value."; 
		}
		
		// uuid type
		else if (type_name eq "uniqueidentifier") {
			xmsortcounter = xmsortcounter + 1; // increment the sort
			node.sortorder.value = xmsortcounter; 
			node.invalidtype.value = "The #column_name# does not contain valid data.  This field must be a valid GUID value.";
		}
		
		// any type other than timestamp
		else if (not listfindnocase("timestamp",type_name)) { // we don't want this in the output
			xmsortcounter = xmsortcounter + 1; // increment the sort
			node.sortorder.value = xmsortcounter; 
			node.invalidtype.value = "The #column_name# does not contain valid data.";
		}
		
		// node opener
		xm.content = listappend(xm.content,'#chr(9)##chr(9)#&lt;#column_name#&gt;',chr(10));
		
		
		for(xi = 1; xi lte listlen(nodelist); xi = xi + 1) { 
			xn = listgetat(nodelist,xi); 
			if(len(node[xn].value) or node[xn].forcetagpair) { // closing and ending tags required?
				xm.content = listappend(xm.content,'#chr(9)##chr(9)##chr(9)#&lt;#node[xn].name#&gt;#node[xn].value#&lt;/#node[xn].name#&gt;',chr(10));
			}
			else { // empty tag
				xm.content = listappend(xm.content,'#chr(9)##chr(9)##chr(9)#&lt;#node[xn].name#/&gt;',chr(10));
			}	
		}
		
		// node closer
		xm.content = listappend(xm.content,'#chr(9)##chr(9)#&lt;/#column_name#&gt;',chr(10));
	</cfscript>
	
</cfoutput>


<cfscript>
	// the final concatenated xml 
	xm.content = htmleditformat(xm.header & chr(10) & xm.content & chr(10) & xm.footer);
</cfscript>

<cfsetting enablecfoutputonly="no">