<cfsetting enablecfoutputonly="yes">

<!--- --------------------------------------------------------------- 
	@copyright: 	(c) 2007 Websolete.com
	
	@author:		Kevin Miller
	@date:		6/9/2007
	@file:		udf_cookieFunctions.cfm 
	
	@description: 	
   		various cookie related functions
		- note that cookie names are forced to uppercase by the routines below
		
	@changelog: 
		[version - date - author - comment]
		1.0.0 - 6/9/2007 - Kevin Miller - creation
------------------------------------------------------------------ --->
	
	<cfscript>
		// thanks to Gary Temme for this compatibility mod: 
		/* ColdFusion 11 dropped the undocumented cfusion_encrypt & decrypt functions while Railo 4.2 still officially supports it.
		This script will reestablish the built-in function. You can also use fusion_encrypt/decrypt (without the "c").
		Based on 2005 blog post: http://www.barneyb.com/barneyblog/2005/10/28/cfusion_encryptcfusion_decrypt-udfs/ */
		function fusion_binaryXOR(n1, n2){
			n1 = formatBaseN(n1, 2);
			n2 = formatBaseN(n2, 2);
			return inputBaseN(replace(n1 + n2, 2, 0, "all"), 2);
		}
		function fusion_encrypt(string, key){
			var i = 0;
			var result = [];
			key = repeatString(key, ceiling(len(string) / len(key)));
			for (i=1;i LTE len(string);i=i+1) {
				arrayAppend(result,  rJustify(formatBaseN(fusion_binaryXOR(asc(mid(string, i, 1)), asc(mid(key, i, 1))), 16), 2));
			}
			return ucase(replace(arrayToList(result,""), " ", "0", "all"));
		}
		function fusion_decrypt(string, key){
			var i = 0;
			var result = [];
			key = repeatString(key, ceiling(len(string) / 2 / len(key)));
			for (i=2;i LTE len(string);i=i+2) {
				arrayAppend(result, chr(fusion_binaryXOR(inputBaseN(mid(string, i - 1, 2), 16), asc(mid(key, i / 2, 1)))));
			}
			return arraytoList(result,"");
		}
		if (NOT StructKeyExists(Server, "Railo") AND NOT StructKeyExists(Server, "Lucee") AND VAL(ListFirst(Server.ColdFusion.ProductVersion)) GTE 11){
			cfusion_encrypt = fusion_encrypt;
			cfusion_decrypt = fusion_decrypt;
		}
	</cfscript>
	
	<cffunction name="getCookie" hint="Retrieves and decrypts encrypted cookie values; if coookie value fails validation, it is deleted" access="public" output="no" returntype="string">
		<cfargument name="name" required="yes" type="string">
		<cfargument name="key" required="yes" type="string">
		
		<cfset var temp = "">
		<cfset var hashvalue = hash(ucase(arguments.name) & arguments.key)>
		
		<cfif isdefined("cookie.#ucase(arguments.name)#")>
			<cfscript>
				temp = urgDecode(cookie[ucase(arguments.name)],arguments.key);
				if(left(temp,len(hashvalue)) eq hashvalue) { // hashvalue must have been prepended
					temp = replacenocase(temp,hashvalue,''); // remove the hashvalue before continuing 
					return temp; // return the cookie's value
				}
				else { // cookie is invalid, outdated format, or has been tampered with
					purgeCookie(arguments.name); // delete the cookie				
				}
			</cfscript>
		<cfelse>
			<cfthrow type="InvalidCookie" message="The cookie named '#ucase(arguments.name)#' does not exist.">
		</cfif>

	</cffunction>
	
	<cffunction name="setCookie" hint="Sets encrypted cookie values" access="public" output="no" returntype="void">
		<cfargument name="name" required="yes" type="string">
		<cfargument name="value" required="yes" type="string">
		<cfargument name="key" required="yes" type="string">
		<cfargument name="expiration" required="no" type="string" default="NEVER"><!--- SESSION, NOW, NEVER, or NUMBER OF DAYS to use to create an expiration value --->
		<cfargument name="httpOnly" required="no" type="boolean" default="false"><!--- when true, cookie is set via cfheader and contains the HttpOnly attribute (cookie not readable via script) --->
		
		<cfset var hashvalue = hash(ucase(arguments.name) & arguments.key)><!--- used as checksum, prepended to value --->
		<cfset var cookieValue  = "#urgEncode("#hashvalue##arguments.value#",arguments.key)#">
		<cfset var expires = "">
		<cfset var ho = "#iif(arguments.httpOnly,de("; HttpOnly"),de(""))#">
		
		<cfif isnumeric(arguments.expiration)>
			<cfset expires = "expires=#dateformat(dateadd('d',arguments.expiration,now()), 'ddd, dd mmm yyyy')# #timeformat(now(), 'HH:mm:ss')# GMT;">
			
		<cfelseif arguments.expiration eq "session">
			<cfset expires = "">
			
		<cfelseif arguments.expiration eq "now">
			<cfset purgeCookie(arguments.name)>
			<cfreturn>
		
		<cfelse><!--- never, equivalent to 30 years --->
			<cfset expires = "expires=#dateformat(dateadd('yyyy',30,now()), 'ddd, dd mmm yyyy')# #timeformat(now(), 'HH:mm:ss')# GMT;">
			
		</cfif>
		
		<cfif arguments.httpOnly>
			<!--- currently must use cfheader to set HttpOnly cookies --->
			<cfheader name="Set-Cookie" value="#ucase(arguments.name)#=#cookieValue#;#expires#path=/#ho#">
			
		<cfelse>
			<cfif arguments.expiration eq "SESSION">
				<cfcookie name="#ucase(arguments.name)#" value="#cookieValue#">
			<cfelse>
				<cfcookie name="#ucase(arguments.name)#" value="#cookieValue#" expires="#arguments.expiration#">
			</cfif>
			
		</cfif>
		
	</cffunction>

	<cffunction name="purgeCookie" hint="Deletes a cookie" access="public" output="no" returntype="void">		
		<cfargument name="name" required="yes" type="string">
		
		<cfcookie name="#ucase(arguments.name)#" expires="NOW" value="0">
	</cffunction>

	<cffunction name="urgEncode" hint="Encodes plain text into an obfuscated, and url safe, string; decode with urgDecode" access="public" output="no" returntype="string">
		<cfargument name="theString" type="string" required="yes"><!--- the string to encode --->
		<cfargument name="theKey" type="string" required="yes"><!--- the key to use for encryption --->
		<cfargument name="theMode" type="string" required="no" default="hard"><!--- hard = encrypt value, soft = pseudo-escape non alphanumeric chars, url = urlencoding only, none = no modification --->
		
		<cfset var work = "">
		
		<cfif arguments.theMode eq "none">
			<cfreturn arguments.theString>
		<cfelseif arguments.theMode eq "url">
			<cfreturn urlencodedformat(arguments.theString)>
		<cfelseif arguments.theMode eq "soft">
			<cfset work = arguments.theString>
			<cfset work = replacenocase(work,"&amp;","|","all")><!--- urlencoded ampersands --->
			<cfset work = replacenocase(work," ","+","all")><!--- spaces --->
			<cfset work = replacenocase(work,",","`","all")><!--- commas --->
			<cfreturn work>
		<cfelse>
			<cfreturn cfusion_encrypt(arguments.theString,theKey)>
		</cfif>
			
	</cffunction>
	
	<cffunction name="urgDecode" hint="Decodes urgEncoded text into plain text" access="public" output="no" returntype="string">
		<cfargument name="theString" type="string" required="yes"><!--- the string to encode --->
		<cfargument name="theKey" type="string" required="yes"><!--- the key to use for encryption --->
		<cfargument name="theMode" type="string" required="no" default="hard"><!--- hard = encrypt value, soft = pseudo-escape non alphanumeric chars, none = no modification --->
		
		<cfset var work = "">
		
		<cfif not len(arguments.theString)>
			<cfreturn "">
		</cfif>
		
		<cfif arguments.theMode eq "none">
			<cfreturn arguments.theString>
		<cfelseif arguments.theMode eq "url">
			<cfreturn urldecode(arguments.theString)>
		<cfelseif arguments.theMode eq "soft">
			<cfset work = right(arguments.theString,len(arguments.theString))>
			<cfset work = replacenocase(work,"|","&amp;","all")><!--- urlencoded ampersands --->
			<cfset work = replacenocase(work,"+"," ","all")><!--- spaces --->
			<cfset work = replacenocase(work,"`",",","all")><!--- commas --->
			<cfreturn work>			
		<cfelse>
			<cfreturn cfusion_decrypt(right(arguments.theString,len(arguments.theString)),theKey)>
		</cfif>	
		
	</cffunction>	
	
<cfsetting enablecfoutputonly="no">


	