<cfsetting enablecfoutputonly="yes" requesttimeout="600">
<!--- 
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
Details/Copyright 

  Name:		 	DBX  - SQLServer / CF Development Tool 
  Author:		Kevin J. Miller (kmiller@websolete.com)
  URL:			http://www.websolete.com
  Copyright  2002-2018 Kevin J. Miller.  Please see terms of use below.
 
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
Version Notes 
 
See readme/version_history.txt file.  

~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
Description 

DBX is a CF application which interacts with SQLServer 2000 / 2005 / 2008 database objects 
and provides an easy way to explore database objects and metadata, as well as 
generate relevant ColdFusion code.  

Will generate cut and paste CFQUERY and CFSTOREDPROC code based on the
datatypes contained in the given db object, and allow the viewing of View, Stored Procedure and 
User Defined Function schemata.

When generating stored procedure (both cfquery and cfstoredproc) code, the code generator 
generates 'positionally correct' SQL statements, in addition to providing the named parameters.
This allows the generated code to be used with earlier versions of CFMX / SQLServer where named 
parameters were not supported, but relied upon sequence of the parameters in the executed call.

~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
Usage 

Edit CONFIG.CFM (in this directory) to configure DBX variables.

Drop the directory anywhere.  A few configuration variables, found in ***config.cfm*** in the root of the 
application, need your attention.  Note that DBX will work 'out of the box' without you changing any 
variables, but it is highly recommended for you to change the password at the very least (the login 
password by default is 'dbx').  

	Login Password: [string] (default: dbx)
	-----------------------
	The login password defaults to "dbx".  Change that to whatever you wish, with empty 
	string "" signifying unrestricted access to dbx (having no password is only recommended for intranet or 
	fully protected - HTTP auth for example - locations). 

		
	IP Masking: [string list] (default: empty)
	-----------------------
	The IP mask list defaults to empty string (no access restrictions).  You may optionally enter in a comma delimited 
	list of IP address masks to restrict DBX access to specified IP addresses or ranges.  When defined, access attempts 
	that do not match the patterns are rejected. Use * for wildcards.  Valid examples are: 121.22.223.14 or 121.22.*.*, 
	and you may specify multiple masks in the list: e.g., 127.0.0.1,192.*.*.*,172.10.100.*  etc.
	
	
	Guest Mode: [boolean] (default: false)
	------------------
	This is a simple true/false variable that allows a more restricted feature set to be used.  When this
	var is set to true (Guest Mode is ON), there is no DSN drop down on the main login page (user's must enter in 
	the DSN name manually), and other 'sensitive' features are unavailable as well.
	
	
	Allow Execute SQL: [boolean] (default: false)
	------------------
	This is a simple true/false variable that denotes whether the Exec SQL window is available; this window 
	is a simple textarea that allows SQL to be executed against the current datasource. 
	
	This var is only valid when Guest mode is false, is on by default.
	
	
	Include Master Objects: [boolean] (default: false)
	---------------------------------
	When this var is set to true, system objects from the Master database will be included in the tree and 
	can be explored.  Please note that this can add a considerable amount of time to the load and 
	performance of the nav tree, since there are typically a huge number of objects in the master db.
	
	Beyond this, there is the fundamental issue of making it easy to explore something that 
		a) you shouldn't even be in the first place (the master db's objects), and
		b) gives you (and others) enough information to, well, put it bluntly, really fuck things up.
		
	The success of being able to access the master db will of course be based on the permissions the 
	user account for the currently active DSN has.  Nevertheless, you have been warned.
		
	This var is only valid when Guest mode is false, and is on by default.
	
	
	Debugging: [boolean] (default: false)
	----------------
	If you have debugging on for this server and the current IP, you can simply set this var to false in 
	order to suppress the debug output (screws up the nav tree for one, pretty unnecessary for this 
	app in general).
	
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
Requirements 

 - CFMX  6.x or higher
 - SQLServer 2000 / 2005 / 2008
 - IE 7+ (may work acceptably in IE 6)
 - FF 1.4+ 
 
Currently a bit wonky in other browsers like Opera and Safari, but no specific work has been done 
to address the issues (little incentive).  Recommended to use one of the browsers listed above. 

Requires Javascript enabled.  Very simple JS is used in the interface (copy to clipboard, 
window.open, etc.).  

Works with both Java native database drivers as well as ODBC connections. 

~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
Notes 

DBX does not use an application scope, nor does it use sessions.  It uses cookies to keep login 
status and to record certain configuration information, and recursively calls its 'main' template in 
order to create its own 'application' type variables, which you define below.  As such, it is safe 
to deploy anywhere, either in its own top level directory or as a subdirectory of another application.

Sets browser cookies (expiry upon exiting browser) with the dsn and login 
info for the duration of the session, so cookies must be enabled.  Additionally, a cookie can be 
set on the initial login screen to 'remember' the last DSN selected from the drop down.

With the inclusion of the master db object viewer (assuming you have enabled Master Objects), 
it is highly recommended that you protect this page via .htaccess or NT permissions at the physical 
file level in addition to setting the password for access to DBX.

Thanks to Mike Wokasch for his help on implementing the code colorizer.

~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
Terms of Use 

Free for commercial and non-commercial use.  You may remove these 
notes but must leave the top detail/copyright header.  If you make 
improvements or would like to see specific features or changes, please 
email me at: kmiller@websolete.com

~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ --->

<!--- SET CONFIGURATION VARIABLES IN THE FOLLOWING TEMPLATE --->
<cfinclude template="config.cfm">

<!--- PLEASE DO NOT ALTER BELOW THIS LINE --->
<cfinclude template="includes/core/appversion.cfm">
		
<!--- PROC PREFIX FILTER: 
		this is used when rendering the left-hand proc tree; 
		SQL2005 treats some non-user procs as user procs (or at least in the context of how DBX retrieves them); 
		a solution to filter them out based on internal metadata has escaped me to date, so this list of prefix filters 
		addresses the issue in a somewhat rudimentary manner; 
		value is a comma delimited list of prefix chars whose procs will be filtered out from display in the tree --->
				
<cfset dbx.procprefixfilter = "dt_">

<!--- dbx's 'request' scope --->
<cfset dbxr = structnew()>

<cfsetting enablecfoutputonly="no">

<cfinclude template="includes/core/i_primary.cfm">

<cfif not getCookie("dbxlogin",dbx.enckey) eq "loggedin" or getCookie("dbxdsn",dbx.enckey) eq "0" or (not isdefined("dbxr.loggedin") or not dbxr.loggedin) or isdefined("url.lo") or isdefined("url.sw")>

	<cfif not getCookie("dbxlogin",dbx.enckey) eq "loggedin" or (not isdefined("dbxr.loggedin") or not dbxr.loggedin) or isdefined("url.lo")>
		<cfinclude template="includes/core/i_login.cfm">
	
	<cfelseif getCookie("dbxdsn",dbx.enckey) eq "0" or isdefined("url.sw")>
		<cfif not len(dbxr.message)>
			<cfset dbxr.message = dbxr.message & "Enter #replace(dbx.supportedDBMS,",","/","all")# DSN information:">
		</cfif>
		<cfinclude template="includes/core/i_getdatasources.cfm">
		<cfinclude template="includes/core/i_dsnselection.cfm">
	</cfif>
	
	<cfif not dbx.debugging>
		<cfsetting showdebugoutput="no">
	</cfif>
	<!--- <cfabort> --->
	
</cfif>


<cftry>

<cfif not isdefined("url.frame")><!--- frameset --->

	<!--- do the code view window --->
	<cfif isdefined("url.cv")>
		<cfinclude template="includes/core/i_codeview.cfm">
	</cfif>
	
	<!--- do the ext prop window --->
	<cfif isdefined("url.aep")>
		<cfinclude template="includes/#dbx.currentDBMS#/i_extpropmanager.cfm">
	</cfif>
	
	<!--- do the create insert window --->
	<cfif isdefined("url.ci") and isdefined("url.t") and len(url.t)>
		<cfinclude template="includes/#dbx.currentDBMS#/i_createinserts.cfm">
	</cfif>
	
	<!--- do the sql exec window --->
	<cfif isdefined("url.se") and not dbx.guestmode>
		<cfinclude template="includes/core/i_sqlexecwindow.cfm">
	</cfif>
	
	<!--- do the object search window --->
	<cfif isdefined("url.search") and not dbx.guestmode>
		<cfinclude template="includes/#dbx.currentDBMS#/i_searchobjects.cfm">
	</cfif>
	
	<!--- do the tuning manager window --->
	<cfif isdefined("url.tuning") and not dbx.guestmode>
		<cfinclude template="includes/#dbx.currentDBMS#/tuning/i_tuningmanager.cfm">
	</cfif>
	
	<cfinclude template="includes/core/i_mainframeset.cfm">
	
<cfelse><!--- do a frame --->

	<cfif url.frame eq "top">
		<cfinclude template="includes/core/i_topnavframe.cfm">
	
	<cfelseif url.frame eq "status">
		<cfinclude template="includes/core/i_bottomnavframe.cfm">
	
	<cfelseif url.frame eq "detail">
	
		<cfif isdefined("url.cfg") and not dbx.guestmode><!--- show server config --->
		
			<cfinclude template="includes/#dbx.currentDBMS#/i_getdbobject.cfm">
			<cfinclude template="includes/#dbx.currentDBMS#/i_serverconfig.cfm">

		<cfelseif isdefined("url.dbcfg") and not dbx.guestmode><!--- show database config --->
		
			<cfinclude template="includes/#dbx.currentDBMS#/i_getdbobject.cfm">
			<cfinclude template="includes/#dbx.currentDBMS#/i_dbconfig.cfm">
			
		<cfelseif isdefined("url.oid")><!--- an object id --->
		
			<cfif not isdefined("url.subframe")><!--- frameset --->
				<cfinclude template="includes/#dbx.currentDBMS#/i_subframeset.cfm">
				
			<cfelseif isdefined("url.codegen")><!--- bottom half of detail frameset --->
				<!--- <cfinclude template="includes/#dbx.currentDBMS#/i_getdbobject.cfm"> --->
				<cfinclude template="includes/#dbx.currentDBMS#/i_sourcepage_codegen.cfm">
				
			<cfelse><!--- top half of detail frameset --->
				<cfinclude template="includes/#dbx.currentDBMS#/i_getdbobject.cfm">
				<cfinclude template="includes/#dbx.currentDBMS#/i_sourcepage.cfm">
			
			</cfif>
		
		<cfelseif isdefined("url.sg") and not dbx.guestmode>
		
			<cfinclude template="includes/#dbx.currentDBMS#/i_scriptgenerator.cfm">
			
		<cfelseif isdefined("url.reports")>
			<cfinclude template="includes/#dbx.currentDBMS#/i_reports.cfm">
			
		<cfelseif isdefined("url.search")>
			<cfinclude template="includes/#dbx.currentDBMS#/i_searchobjects.cfm">
		
		<cfelse>
			<cfinclude template="includes/core/i_help_defaults.cfm">
			
		</cfif>
		
	<cfelseif url.frame eq "listheader">
		<cfinclude template="includes/#dbx.currentDBMS#/i_tree_listheader.cfm">
	
	<cfelseif url.frame eq "listmasterheader">
		<cfinclude template="includes/#dbx.currentDBMS#/i_tree_listmasterheader.cfm">
	
	<cfelseif url.frame eq "list">
		<cfinclude template="includes/#dbx.currentDBMS#/i_tree_list.cfm">
		
	<cfelseif url.frame eq "listmaster">
		<cfinclude template="includes/#dbx.currentDBMS#/i_tree_listmaster.cfm">
		
		</cfif>
	
	</cfif>

<cfcatch>
	<cfinclude template="includes/core/i_errordump.cfm">	
	</cfcatch>
</cftry>

<cfif not dbx.debugging>
	<cfsetting showdebugoutput="no">
</cfif>
